/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak;

import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.julia.loader.Loader;
import org.objectweb.fractal.julia.loader.Tree;


/**
 * Juliak bootstrap component implementation (aka content class).
 * This component provides three interfaces:
 * 
 * <ul>
 * <li> <code>type-factory</code> of type {@link TypeFactory} </li>
 * <li> <code>generic-factory</code> of type {@link GenericFactory} </li>
 * <li> <code>loader</code> of type {@link Loader} </li>
 * </ul>
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class BootstrapComponentImpl implements TypeFactory, GenericFactory, Loader {
    
    // -----------------------------------------------------------------
    // Implementation of the TypeFactory interface
    // -----------------------------------------------------------------
    
    private TypeFactory tf = Juliak.get().getFcTypeFactory();
    
    public InterfaceType createFcItfType(
            String name, String signature,
            boolean isClient, boolean isOptional, boolean isCollection)
    throws InstantiationException {
        
        return tf.createFcItfType(
                name,signature,isClient,isOptional,isCollection);        
    }

    public ComponentType createFcType(InterfaceType[] interfaceTypes)
    throws InstantiationException {
        
        return tf.createFcType(interfaceTypes);
    }

    // -----------------------------------------------------------------
    // Implementation of the GenericFactory interface
    // -----------------------------------------------------------------
    
    private GenericFactory gf = Juliak.get().getFcGenericFactory();
    
    public Component newFcInstance(
            Type type, Object controllerDesc, Object contentDesc)
    throws InstantiationException {
        
        return gf.newFcInstance(type,controllerDesc,contentDesc);        
    }

    // -----------------------------------------------------------------
    // Implementation of the Loader interface
    // -----------------------------------------------------------------
    
    private Loader loader = Juliak.get().getFcLoader();

    public Tree evalTree(Tree tree, Map context) throws Exception {
        return loader.evalTree(tree, context);
    }

    public void init(Map context) throws Exception {
        loader.init(context);
    }

    public Class loadClass(String className, Object loader) throws ClassNotFoundException {
        return this.loader.loadClass(className, loader);
    }

    public Class loadClass(Tree classDescriptor, Object loader) throws ClassNotFoundException {
        return this.loader.loadClass(classDescriptor, loader);
    }

    public Tree loadTree(String name) throws Exception {
        return loadTree(name);
    }

    public Object newObject(Tree objectDescriptor, Object loader) throws Exception {
        return this.loader.newObject(objectDescriptor, loader);
    }
    
}
