/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.membrane;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.juliak.platform.Platform;


/**
 * Class holding data for defining a membrane.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class MembraneDef {
    
    /** The controller description (e.g. primitive, composite). */
    private Object controllerdesc;
    
    /**
     * The fully qualified name of the Fractal ADL architecture describing this
     * membrane.
     */
    private String adl;
    
    public MembraneDef( Object controllerdesc, String adl ) {
        this.controllerdesc = controllerdesc;
        this.adl = adl;
    }
    
    public Object getControllerDesc() {
        return controllerdesc;
    }
    
    public String getAdl() {
        return adl;
    }
    
    /**
     * Return a new instance of this membrane, i.e. create an instance of the
     * composite referenced by adl.
     */
    public Component newFcStaticMembrane( TypeFactory tf, GenericFactory gf )
    throws InstantiationException {
        
        try {
            if( adlsf == null ) {
                Class adlcl = Platform.get().loadClass(adl);
                if( ! CustomFactory.class.isAssignableFrom(adlcl) ) {
                    final String msg = 
                        adlcl.getName()+" should implement "+
                        CustomFactory.class.getName();
                    throw new Exception(msg);
                }
                adlsf = (CustomFactory) adlcl.newInstance();
                adlsf.setFcTypeFactory(tf);
                adlsf.setFcGenericFactory(gf);
            }
        
            return adlsf.newFcInstance();
        }
        catch( Exception e ) {
            throw new InstantiationException(e.getMessage());
        }
    }
    
    private CustomFactory adlsf;
}
