/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.platform;

import java.io.IOException;
import java.io.InputStream;

/**
 * Platform specific features.
 *  
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public interface PlatformItf {

    /**
     * Set the loader to be used by Juliak for loading classes.
     * 
     * @param cl  the loader.
     *      This parameter must implement ClassLoader. It is declared as an
     *      Object in order to be compliant with the Java ME CLDC version of
     *      Juliak.
     */
    public void setLoader(Object cl);

    /**
     * Return the loader used by Juliak for loading classes.
     * 
     * @return  the loader.
     *      This parameter must implement ClassLoader. It is declared as an
     *      Object in order to be compliant with the Java ME CLDC version of
     *      Juliak.
     */
    public Object getLoader();

    /**
     * Load a class.
     */
    public Class loadClass(String name) throws ClassNotFoundException;

    /**
     * Load a ressource.
     */
    public InputStream getResourceAsStream(String name);

    /**
     * Define a new class.
     */
    public Class defineClass(String name, byte[] b);

    /**
     * Utility method to write the bytecode from a class to a file. This method
     * creates the directory structure corresponding to package names.
     * 
     * @param dirname    the root directory name
     * @param classname  the name of the class
     * @param b          the bytecode
     */
    public void dumpClassToFile(String dirname, String classname, byte[] b)
    throws IOException;

}